#ifndef LIBWCCL_OPS_FUNCTIONS_BOOL_ITERATION_H
#define LIBWCCL_OPS_FUNCTIONS_BOOL_ITERATION_H

#include <libwccl/ops/function.h>
#include <libwccl/values/bool.h>
#include <libwccl/values/position.h>

namespace Wccl {

/**
 * Abstract base class for a class of operators that iterate over a range.
 */
class Iteration : public Function<Bool> {
public:
	typedef boost::shared_ptr<Function<Position> > PosFunctionPtr;
	typedef boost::shared_ptr<Function<Bool> > BoolFunctionPtr;
	static const int MatchAll = boost::integer_traits<int>::const_min;
	typedef enum Direction
	{
		Forward,
		Backward
	} Direction;

protected:

	Iteration(
		const PosFunctionPtr& range_begin_expr,
		const PosFunctionPtr& range_end_expr,
		const VariableAccessor<Position>& iter_var_acc,
		const BoolFunctionPtr& condition_expr,
		int match_count = MatchAll,
		Direction direction = Forward)
		: range_begin_expr_(range_begin_expr),
		  range_end_expr_(range_end_expr),
		  iter_var_acc_(iter_var_acc),
		  condition_expr_(condition_expr),
		  match_count_(match_count),
		  direction_(direction)
	{
		BOOST_ASSERT(range_begin_expr_);
		BOOST_ASSERT(range_end_expr_);
		BOOST_ASSERT(condition_expr_);
		BOOST_ASSERT(match_count == MatchAll || match_count > 0);
	}

	/**
	 * Evaluate argument expression and assign the result to underlying variable.
	 * @returns True.
	 */
	BaseRetValPtr apply_internal(const FunExecContext& context) const;

	const PosFunctionPtr& range_begin_expr_;
	const PosFunctionPtr& range_end_expr_;
	const VariableAccessor<Position> iter_var_acc_;
	const BoolFunctionPtr& condition_expr_;
	const int match_count_;
	const Direction direction_;

private:
	bool iterate(int start, int stop, int step, Position& p, const FunExecContext& context) const;
};


} /* end ns Wccl */

#endif // LIBWCCL_OPS_FUNCTIONS_BOOL_ITERATION_H
